// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include "lock.h"
#include "dump.h"
#include <math.h>

void save_config(char *simfile,struct device *in)
{

}

void config_read_line_to_double(double *data,FILE *in,char *id)
{
char name[100];
fscanf(in,"%s",name);
fscanf(in,"%le",data);
if (strcmp(name,id)!=0)
{
	printf("Error read %s but expected %s\n",name,id);
	exit(0);
}
}

void config_read_line_to_int(int *data,FILE *in,char *id)
{
char name[100];
fscanf(in,"%s",name);
fscanf(in,"%d",data);
if (strcmp(name,id)!=0)
{
	printf("Error read %s but expected %s\n",name,id);
	exit(0);
}
}

void load_config(char *simfile,struct device *in,int gui)
{
double ver;
int i;
int dump;
FILE *config;
char name[1000];
slice_init(in);
config=fopena(in->outputpath,"device_epitaxy.inp","r");
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->ymeshlayers));

in->meshdata = malloc (in->ymeshlayers * sizeof(struct mesh));

in->ymeshpoints=0;

for (i=0;i<in->ymeshlayers;i++)
{
	fscanf(config,"%lf %lf",&(in->meshdata[i].len),&(in->meshdata[i].number));
	in->meshdata[i].den=in->meshdata[i].len/in->meshdata[i].number;
	in->ymeshpoints+=in->meshdata[i].number;
}

in->imat=malloc (in->ymeshpoints * sizeof(int));

	in->ymesh = malloc (in->ymeshpoints * sizeof(double));

	int pos=0;
	int ii;
	double dpos=0.0;
	for (i=0;i<in->ymeshlayers;i++)
	{

		for (ii=0;ii<in->meshdata[i].number;ii++)
		{
			dpos+=in->meshdata[i].den/2.0;
			in->ymesh[pos]=dpos;
			in->imat[pos]=i;
			dpos+=in->meshdata[i].den/2.0;
			pos++;
		}
	}


fscanf(config,"%d",&(in->mat.number));
get_max_layers(in->mat.number);

in->mat_layers=in->mat.number;
in->mat.l = malloc (in->mat.number * sizeof(struct layer));

for (i=0;i<in->mat.number;i++)
{
	fscanf(config,"%s %le %le",in->mat.l[i].name,&(in->mat.l[i].height),&(in->mat.l[i].Nad));
	if (gui==FALSE) load_mat_file(in->mat.l[i].name,&(in->mat.l[i]));
}


fscanf(config,"%s",name);
fscanf(config,"%lf",&(ver));
if (ver!=1.0)
{
printf("File compatability problem %s\n","device_epitaxy.inp");
exit(0);
}

fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
printf("Problem with device_epitaxy.inp file last item read %s!\n",name);
exit(0);
}

fclose(config);

config=fopena(in->outputpath,simfile,"r");

fscanf(config,"%s",name);
fscanf(config,"%d",&(in->bdos));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->xlen));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->zlen));

in->area=in->xlen*in->zlen;


fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Rshunt));
in->Rshunt=fabs(in->Rshunt);

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Rcontact));
in->Rcontact=fabs(in->Rcontact);	//This is a resistance in ohms

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Rshort));

fscanf(config,"%s",name);
fscanf(config,"%d",&(in->Dphoton));
fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->Dphotoneff));
in->Dphotoneff=fabs(in->Dphotoneff);
fscanf(config,"%s",name);
fscanf(config,"%le ",&(in->lcharge));
in->lcharge=fabs(in->lcharge);
fscanf(config,"%s",name);
fscanf(config,"%le ",&(in->rcharge));
in->rcharge=fabs(in->rcharge);
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->other_layers));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->B));
in->B=fabs(in->B);
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->doping));

fscanf(config,"%s",name);
fscanf(config,"%d",&(in->interfaceleft));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->interfaceright));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->phibleft));
fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->phibright));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->vl_e));
in->vl_e=fabs(in->vl_e);

fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->vl_h));
in->vl_h=fabs(in->vl_h);

fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->vr_e));
in->vr_e=fabs(in->vr_e);

fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->vr_h));
in->vr_h=fabs(in->vr_h);

fscanf(config,"%s",name);
fscanf(config,"%lf",&(ver));
if (ver!=1.0)
{
printf("File compatability problem %s\n",simfile);
exit(0);
}

fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
printf("Problem with device.inp file last item read %s!\n",name);
exit(0);
}
fclose(config);


config=fopena(in->outputpath,"math.inp","r");

fscanf(config,"%s",name);
fscanf(config,"%d",&(in->max_electrical_itt));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->electrical_clamp));

fscanf(config,"%s",name);
fscanf(config,"%d",&(in->max_electrical_itt0));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->electrical_clamp0));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->posclamp));

fscanf(config,"%s",name);
fscanf(config,"%d",&(in->enable_back));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->min_cur_error));

fscanf(config,"%s",name);
fscanf(config,"%d",&(in->newton_clever_exit));

fscanf(config,"%s",name);
fscanf(config,"%d",&(in->newton_min_itt));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(ver));
if (ver!=1.0)
{
printf("File compatability problem %s\n","math.inp");
exit(0);
}

fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
printf("Problem with math.inp file last item read %s!\n",name);
exit(0);
}
fclose(config);


config=fopena(in->outputpath,"thermal.inp","r");

fscanf(config,"%s",name);
fscanf(config,"%d",&(in->newton_enable_external_thermal));

fscanf(config,"%s",name);
fscanf(config,"%d",&(in->nofluxl));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->Tll));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->Tlr));

fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
printf("Problem with thermal.inp file last item read %s!\n",name);
exit(0);
}
fclose(config);


config=fopena(in->outputpath,"sim.inp","r");
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->simmode));

fscanf(config,"%s",name);
fscanf(config,"%d",&(in->Rbi_on));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->Gp_on));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->Gn_on));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->stoppoint));

in->srh_sim=TRUE;
in->ntrapnewton=TRUE;
in->ptrapnewton=TRUE;

fscanf(config,"%s",name);
fscanf(config,"%lf",&(ver));
if (ver!=1.0)
{
printf("File compatability problem %s\n","sim.inp");
exit(0);
}

fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
printf("Problem with sim.inp file last item read %s!\n",name);
exit(0);
}
fclose(config);

//<clean>
config=fopena(in->outputpath,"tpv.inp","r");
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tpvdtstart));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tpvstart));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tpvlaseroff));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tpvlaserpower));
in->tpvlaserpower=fabs(in->tpvlaserpower);
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tpvdtmull));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tpvstop));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tpvvoc));
fscanf(config,"%s",name);
fscanf(config,"%d ",&(in->tpv_mode));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Rscope));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tpv_L));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(ver));
if (ver!=1.0)
{
printf("File compatability problem %s\n","tpv.inp");
exit(0);
}

fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
printf("Problem with tpv.inp file last item read %s!\n",name);
exit(0);
}
fclose(config);
//</clean>


//<clean>
config=fopena(in->outputpath,"tpc.inp","r");
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tpcstop));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tpclaseroff));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tpcdtstart));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tpcdtmull));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tpclaserpower));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Rtpc_short));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->tsteps));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->pulsestart));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->pulselen));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->reprate));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->timeshift));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tpc_externalv));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->tpc_sim_all));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tpc_L));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(ver));
if (ver!=1.0)
{
printf("File compatability problem %s\n","tpc.inp");
exit(0);
}

fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
printf("Problem with tpc.inp file last item read %s!\n",name);
exit(0);
}
fclose(config);
//</clean>

//<clean>
config=fopena(in->outputpath,"wire.inp","r");
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->simwire));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(ver));
if (ver!=1.0)
{
printf("File compatability problem %s\n","wire.inp");
exit(0);
}

fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
printf("Problem with wire.inp file last item read %s!\n",name);
exit(0);
}
fclose(config);
//</clean>

config=fopena(in->outputpath,"dump.inp","r");
fscanf(config,"%s",name);
fscanf(config,"%s",name);

fscanf(config,"%s",name);
fscanf(config,"%d",&(dump));
fscanf(config,"%d",&(dump));
set_dump_status(dump_plot,dump);

fscanf(config,"%s",name);
fscanf(config,"%d",&(dump));
set_dump_status(dump_newton, dump);
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->plottime));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->stop_start));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->dumpitdos));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->slave));
fscanf(config,"%s",name);
fscanf(config,"%s",in->slave_path);
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->master));
fscanf(config,"%s",name);
fscanf(config,"%s",in->plot_file);
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->start_stop_time));


fscanf(config,"%s",name);
fscanf(config,"%d",&(dump));
set_dump_status(dump_iodump,dump);
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->dump_movie));

fscanf(config,"%s",name);
fscanf(config,"%d",&(dump));
set_dump_status(dump_optics, dump);

if (get_dump_status(dump_iodump)==FALSE)
{
in->dumpitdos=FALSE;
set_dump_status(dump_newton, FALSE);
set_dump_status(dump_optics,FALSE);
}

fscanf(config,"%s",name);
fscanf(config,"%d",&(dump));
set_dump_status(dump_energy_slice_switch, dump);

fscanf(config,"%s",name);
fscanf(config,"%d",&(dump));
set_dump_status(dump_print_newtonerror, dump);

fscanf(config,"%s",name);
fscanf(config,"%d",&(dump));
set_dump_status(dump_print_converge, dump);

fscanf(config,"%s",name);
fscanf(config,"%lf",&(ver));
if (ver!=1.0)
{
printf("File compatability problem %s\n",simfile);
exit(0);
}

fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
printf("Problem with dump.inp file last item read %s!\n",name);
exit(0);
}
fclose(config);


config=fopena(in->outputpath,"jv.inp","r");

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Vstart));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Vstop));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Vstep));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->n_id));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Is));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Igen));


fscanf(config,"%s",name);
fscanf(config,"%lf",&(ver));
if (ver!=1.0)
{
printf("File compatability problem %s\n","jv.inp");
exit(0);
}

fscanf(config,"%s",name);

if (strcmp("#end",name)!=0)
{
printf("Problem with jv.inp file last item read %s!\n",name);
exit(0);
}
fclose(config);

//<clean>
config=fopena(in->outputpath,"celiv.inp","r");

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->celiv_dt));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->celiv_t0));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->celiv_t1));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->celiv_t2));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->celiv_Vexternal));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->celiv_Vmax));

fscanf(config,"%s",name);
fscanf(config,"%d",&(in->celiv_repeat));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->celiv_time_j0));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->celiv_time_j1));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Rshort_celiv));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->celiv_laser_on));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->celiv_laser_width));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->celiv_laser_power));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->celiv_Vss));

fscanf(config,"%s",name);
fscanf(config,"%ld",&(in->celiv_start_at_vbi));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(ver));
if (ver!=1.0)
{
printf("File compatability problem %s\n","celiv.inp");
exit(0);
}

fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
printf("Problem with celiv.inp file last item read %s!\n",name);
exit(0);
}
fclose(config);
//</clean>

config=fopena(in->outputpath,"pulse.inp","r");

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->pulse_dt));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->pulse_t0));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->pulse_t1));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->pulse_t2));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->pulse_Vexternal));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->pulse_Vmax));

fscanf(config,"%s",name);
fscanf(config,"%d",&(in->pulse_repeat));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->pulse_time_j0));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->pulse_time_j1));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Rshort_pulse));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->pulse_laser_on));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->pulse_laser_width));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->pulse_laser_power));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->pulse_Vss));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->pulse_light));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->pulse_sun));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(ver));
if (ver!=1.0)
{
printf("File compatability problem %s\n","pulse.inp");
exit(0);
}

fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
printf("Problem with pulse.inp file last item read %s!\n",name);
exit(0);
}
fclose(config);

//<clean>
config=fopena(in->outputpath,"tof.inp","r");

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tofstop));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->toflaseroff));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tofdtstart));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->tofdtmull));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->toflaserpower));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(ver));
if (ver!=1.0)
{
printf("File compatability problem %s\n","tof.inp");
exit(0);
}

fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
printf("Problem with tof.inp file last item read %s!\n",name);
exit(0);
}
fclose(config);
//</clean>


config=fopena(in->outputpath,"fish.inp","r");

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->fish_tstart));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->fish_1start));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->fish_1stop));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->fish_2start));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->fish_2stop));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->fish_den));
in->fish_den=fabs(in->fish_den);
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->fish_externalv));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->fish_dt));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->fish_dt_mull));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->fish_stop));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->fish_stark));
in->fish_stark=fabs(in->fish_stark);
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->fish_slicepos));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(ver));
if (ver!=1.0)
{
printf("File compatability problem %s\n","celiv.inp");
exit(0);
}

fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
printf("Problem with tof.inp file last item read %s!\n",name);
exit(0);
}
fclose(config);

//<clean>
config=fopena(in->outputpath,"ce.inp","r");

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->cedtmull));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->cetimestop));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->cevoltage));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->ceshortvoltage));
fscanf(config,"%s",name);
fscanf(config,"%d ",&(in->cefast));
fscanf(config,"%s",name);
fscanf(config,"%le ",&(in->cedt));
fscanf(config,"%s",name);
fscanf(config,"%d ",&(in->cepresteps));

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->ce_L));


fscanf(config,"%s",name);
fscanf(config,"%lf",&(ver));
if (ver!=1.0)
{
printf("File compatability problem %s\n","ce.inp");
exit(0);
}

fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
printf("Problem with ce.inp file last item read %s!\n",name);
exit(0);
}
fclose(config);
//</clean>

//<clean>
config=fopena(in->outputpath,"li_sc.inp","r");

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->li_sc_sunstart));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->li_sc_sunstop));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->li_sc_sundp));
fscanf(config,"%s",name);
fscanf(config,"%le",&(in->li_sc_sundpmul));

fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
printf("Problem with li_sc.inp file last item read %s!\n",name);
exit(0);
}
fclose(config);
//</clean>

config=fopena(in->outputpath,"light.inp","r");

fscanf(config,"%s",name);
fscanf(config,"%le",&(in->Psun));
fscanf(config,"%s",name);
fscanf(config,"%d",&(in->laser));
fscanf(config,"%s",name);
fscanf(config,"%lf",&(in->fgen));

fscanf(config,"%s",name);
fscanf(config,"%lf",&(ver));
if (ver!=1.0)
{
printf("File compatability problem %s\n","light.inp");
exit(0);
}

fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
printf("Problem with light.inp file last item read %s!\n",name);
exit(0);
}
fclose(config);

in->ylen=0.0;
for (i=0;i<in->mat.number;i++)
{
in->ylen+=in->mat.l[i].height;
}

}
