// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <errno.h>
#include "sim.h"
#include "sun.h"
#include "complex_solver.h"

void light_load_config(struct light *in,char *output_path)
{
char temp[200];

strcpy(in->output_path,output_path);
sprintf(in->config_file,"%soptics.inp",in->output_path);
printf("Optics config file %s\n",in->config_file);
FILE *config=fopen(in->config_file,"r");
fscanf(config,"%s",temp);
fscanf(config,"%s",temp);
FILE *epi=fopena(in->output_path,temp,"r");
int i=0;
double pos=0.0;

	fscanf(config,"%s",temp);
	fscanf(config,"%s",temp);
	inter_load_a(&(in->sun_read),temp,0,1);
	inter_sort(&(in->sun_read));

	fscanf(epi,"%s",temp);
	fscanf(epi,"%d",&in->layers);
	in->thick=(double *)malloc(in->layers*sizeof(double));
	in->G_percent=(double *)malloc(in->layers*sizeof(double));
	in->enable_gen=(int *)malloc(in->layers*sizeof(int));
	in->mat=(struct istruct *)malloc(in->layers*sizeof(struct istruct));
	in->mat_n=(struct istruct *)malloc(in->layers*sizeof(struct istruct));
	in->laser_photon_density=-1.0;
	in->ylen=0.0;
	in->device_start=0.0;
	double mul=1.0;
	for (i=0;i<in->layers;i++)
	{
		fscanf(epi,"%s",temp);
		fscanf(epi,"%le",&(in->thick[i]));
		//printf("%le\n",in->thick[i]);

		fscanf(epi,"%s",temp);
		inter_load_a(&(in->mat[i]),temp,0,1);
		inter_sort(&(in->mat[i]));

		fscanf(epi,"%s",temp);
		inter_load_a(&(in->mat_n[i]),temp,0,1);
		inter_sort(&(in->mat_n[i]));
		int device;
		fscanf(epi,"%d",&device);
		if (device==FALSE)
		{
		in->device_start+=in->thick[i];
		in->device_start_layer=i;
		}
		fscanf(epi,"%d",&(in->enable_gen[i]));
		fscanf(epi,"%le",&(mul));
		inter_mul(&(in->mat[i]),mul);
		in->ylen+=in->thick[i];
	}

	double ver;
	fscanf(epi,"%s",temp);
	fscanf(epi,"%lf",&(ver));

	if (ver!=1.0)
	{
	printf("File compatability problem %s\n","optics_epitaxy.inp");
	exit(0);
	}

	fscanf(epi,"%s",temp);
	if (strcmp("#end",temp)!=0)
	{
	printf("Problem with device_epitaxy.inp file last item read %s!\n",temp);
	exit(0);
	}

	fscanf(config,"%s",temp);
	fscanf(config,"%d",&in->align_mesh);

	fscanf(config,"%s",temp);
	fscanf(config,"%d",&in->points);

	if (in->align_mesh==FALSE)
	{
		in->dx=in->ylen/((double)in->points);
	}else
	{
		in->points=(int)(in->ylen/in->dx);
	}

	fscanf(config,"%s",temp);
	fscanf(config,"%d",&in->lpoints);



	in->x=(double *)malloc(in->points*sizeof(double));
	in->H1d=(double *)malloc(in->points*sizeof(double));
	in->l=(double *)malloc(in->points*sizeof(double));
	in->Gn=(double *)malloc(in->points*sizeof(double));
	in->Gp=(double *)malloc(in->points*sizeof(double));
	in->sun=(double *)malloc(in->lpoints*sizeof(double));
	in->layer_end=(double *)malloc(in->points*sizeof(double));
	in->sun_norm=(double *)malloc(in->lpoints*sizeof(double));
	in->sun_photons=(double *)malloc(in->lpoints*sizeof(double));
	in->sun_E=(double *)malloc(in->lpoints*sizeof(double));
	in->En=(double **)malloc(in->lpoints*sizeof(double*));
	in->Enz=(double **)malloc(in->lpoints*sizeof(double*));
	in->Ep=(double **)malloc(in->lpoints*sizeof(double*));
	in->Epz=(double **)malloc(in->lpoints*sizeof(double*));
	in->photons_asb=(double **)malloc(in->lpoints*sizeof(double*));
	in->H=(double **)malloc(in->lpoints*sizeof(double*));
	in->alpha=(double **)malloc(in->lpoints*sizeof(double*));
	in->photons=(double **)malloc(in->lpoints*sizeof(double*));
	in->n=(double **)malloc(in->lpoints*sizeof(double*));
	in->t=(double complex **)malloc(in->lpoints*sizeof(double complex *));
	in->r=(double complex **)malloc(in->lpoints*sizeof(double complex *));

	for (i=0;i<in->lpoints;i++)
	{
		in->En[i]=(double *)malloc(in->points*sizeof(double));
		in->Enz[i]=(double *)malloc(in->points*sizeof(double));
		in->Ep[i]=(double *)malloc(in->points*sizeof(double));
		in->Epz[i]=(double *)malloc(in->points*sizeof(double));
		in->photons_asb[i]=(double *)malloc(in->points*sizeof(double));
		in->alpha[i]=(double *)malloc(in->points*sizeof(double));
		in->photons[i]=(double *)malloc(in->points*sizeof(double));
		in->n[i]=(double *)malloc(in->points*sizeof(double));
		in->t[i]=(double complex *)malloc(in->points*sizeof(double complex));
		in->r[i]=(double complex *)malloc(in->points*sizeof(double complex));
		in->H[i]=(double complex *)malloc(in->points*sizeof(double complex));
	}

	in->layer=(int *)malloc(in->points*sizeof(int));

	pos=in->dx/2.0;

	int layer=0;
	double layer_end=in->thick[layer];
	//printf("%le\n",layer_end);
	for (i=0;i<in->points;i++)
	{
		in->x[i]=pos;
		in->layer_end[i]=layer_end-pos;
		in->layer[i]=layer;
		if (in->device_start_layer>=layer) in->device_start_i=i;
		//printf("%d %d %d %d\n",in->device_start_i,in->points,in->device_start_layer,layer);
		pos+=in->dx;

		if (pos>layer_end)
		{
			//printf("%le\n",in->thick[layer],la);
			//do
			//{
			if (layer<(in->layers-1))
			{
				layer++;
			//}while(in->thick[layer]==0.0);

				layer_end=layer_end+in->thick[layer];
			}
		}

		//printf("%le %d %d %le\n",in->x[i],i,layer,in->thick[layer]);
	}
	in->device_start_i++;
//exit(0);

	fscanf(config,"%s",temp);
	fscanf(config,"%le",&in->lstart);

	fscanf(config,"%s",temp);
	fscanf(config,"%le",&in->lstop);
	in->dl=(in->lstop-in->lstart)/((double)in->lpoints);

	fscanf(config,"%s",temp);
	fscanf(config,"%le",&in->laser_wavelength);
	in->laser_pos=(int)((in->laser_wavelength-in->lstart)/in->dl);

	fscanf(config,"%s",temp);
	fscanf(config,"%le",&in->spotx);

	fscanf(config,"%s",temp);
	fscanf(config,"%le",&in->spoty);

	fscanf(config,"%s",temp);
	fscanf(config,"%le",&in->pulseJ);

	fscanf(config,"%s",temp);
	fscanf(config,"%le",&in->pulseL);

	fscanf(config,"%s",temp);
	fscanf(config,"%le",&in->lasereff);

	//printf("%d %le\n",in->laser_pos,(in->laser_wavelength-in->lstart)/in->dl);
//exit(0);
	pos=in->lstart;
	for (i=0;i<in->lpoints;i++)
	{
		in->l[i]=pos;
		pos+=in->dl;
	}

	int ii;
	double nl=0.0;
	double nc=0.0;
	double kl=0.0;
	double kc=0.0;
	double complex n0=0.0+0.0*I;
	double complex n1=0.0+0.0*I;
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			in->alpha[i][ii]=inter_get_noend(&(in->mat[in->layer[ii]]),in->l[i]);
			in->n[i][ii]=inter_get_noend(&(in->mat_n[in->layer[ii]]),in->l[i]);
			if (ii==0)
			{
				nl=in->n[i][ii];
				kl=in->alpha[i][ii]*(in->l[i]/(pi*2.0));
			}else
			{
				nl=in->n[i][ii-1];
				kl=in->alpha[i][ii-1]*(in->l[i]/(pi*2.0));
			}
			nc=in->n[i][ii];
			kc=in->alpha[i][ii]*(in->l[i]/(pi*2.0));

			n0=nl+kl*I;
			n1=nc+kc*I;

			in->r[i][ii]=(n1-n0)/(n0+n1);
			in->t[i][ii]=(2.0*n1)/(n0+n1);

			in->En[i][ii]=1.0;
			in->Ep[i][ii]=1.0;
			in->Enz[i][ii]=1.0;
			in->Epz[i][ii]=1.0;

		}
	}

	for (i=0;i<in->lpoints;i++)
	{
		in->sun_norm[i]=inter_get_noend(&(in->sun_read),in->l[i]);
	}

	double tot=0.0;
	for  (i=0;i<in->lpoints;i++)
	{
		tot+=in->dl*in->sun_norm[i];
	}

	for  (i=0;i<in->lpoints;i++)
	{
		in->sun_norm[i]/=tot;
	}

	fscanf(config,"%s",temp);
	fscanf(config,"%le",&(in->ND));

	fscanf(config,"%s",temp);
	fscanf(config,"%d",&(in->gather));

	fscanf(config,"%s",temp);
	fscanf(config,"%le",&(in->dlength));

	fscanf(config,"%s",temp);
	fscanf(config,"%lf",&(in->Eg));

	fscanf(config,"%s",temp);
	fscanf(config,"%d",&(in->inverted));

	fscanf(config,"%s",temp);
	fscanf(config,"%le",&in->electron_eff);

	fscanf(config,"%s",temp);
	fscanf(config,"%le",&in->hole_eff);

	fscanf(config,"%s",temp);
	fscanf(config,"%lf",&(ver));

	if (ver!=1.0)
	{
	printf("File compatability problem %s\n","optics_epitaxy.inp");
	exit(0);
	}

fscanf(config,"%s",temp);
if (strcmp(temp,"#end")!=0)
{
	printf("Error at end of lightfile file\n");
	exit(0);
}

fclose(epi);
fclose(config);


in->N=0.0;
in->N+=in->points*2-1;
in->N+=in->points*2-1;
in->N+=in->points;
in->N+=in->points;
//in->N+=1;
in->M=in->points+in->points;
in->Ti=malloc(in->N*sizeof(int));
in->Tj= malloc(in->N*sizeof(int));
in->Tx=malloc(in->N*sizeof(double));
in->Txz=malloc(in->N*sizeof(double));
in->b=malloc(in->M*sizeof(double));
in->bz=malloc(in->M*sizeof(double));

}
