// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "sim.h"
#include "dump.h"

void sim_pulse(struct device *in)
{
//<clean>
in->dt=in->pulse_dt;

carrier_count_reset(in);

in->go_time=FALSE;


//struct istruct pulseout;
//inter_init(&pulseout);
in->externalv=in->pulse_Vexternal;
printf("Running pulse. %e\n",in->externalv);
int ittr=0;

int step=0;
solve_light(in,&(in->mylight),in->Psun,0.0);

//in->Vapplied=0.1;
//ramp(in,in->Vapplied,in->pulse_Vmax,100.0);
//sim_externalv(in,in->pulse_Vmax);
//double pulse_max=in->pulse_Vmax;

//ramp(in,in->Vapplied,in->pulse_Vexternal,100.0);
sim_externalv(in,in->pulse_Vexternal);
in->Vapplied=in->pulse_Vexternal;
double V=in->Vapplied;
double V0=in->pulse_Vexternal;
printf("%le %le\n",V,V0);
solve_light(in,&(in->mylight),in->Psun,0.0);

newton_celiv(in,V0,FALSE);
device_timestep(in);
newton_celiv(in,V0,FALSE);
device_timestep(in);
in->time=0.0;

time_init(in);

device_timestep(in);

in->go_time=TRUE;


//double np_out_tot=0.0;
double t0=in->pulse_t0;
double t1=in->pulse_t0+in->pulse_t1;
double t2=in->pulse_t0+in->pulse_t1+in->pulse_t2;
FILE *pulse_out=fopen("./pulse_j.dat","w");
fclose(pulse_out);

pulse_out=fopen("./pulse_v.dat","w");
fclose(pulse_out);

pulse_out=fopen("./pulse_nleft.dat","w");
fclose(pulse_out);

pulse_out=fopen("./pulse_rtot.dat","w");
fclose(pulse_out);

double local_time=in->time;

struct istruct out_j;
inter_init(&out_j);

struct istruct out_v;
inter_init(&out_v);

struct istruct out_mue;
inter_init(&out_mue);

struct istruct out_muh;
inter_init(&out_muh);

struct istruct out_mue_c;
inter_init(&out_mue_c);

struct istruct out_muh_c;
inter_init(&out_muh_c);

struct istruct out_G;
inter_init(&out_G);

struct istruct lost_charge;
inter_init(&lost_charge);

double mu_e_start=get_avg_mue(in);
double mu_h_start=get_avg_muh(in);

//	printf("%le %le\n",in->Vapplied,in->Vapplied_last);
	//exit(0);
double extracted_through_contacts=0.0;
//reset_npinit(in);
do
{


	if (local_time<t0)
	{
		if (in->pulse_light==TRUE)
		{
			solve_light(in,&(in->mylight),0.0,0.0);
		}else
		{
			V=V0;

			if ((local_time>=in->pulse_laser_on)&&(local_time<=(in->pulse_laser_on+in->pulse_laser_width)))
			{
				solve_light(in,&(in->mylight),in->Psun,in->pulse_laser_power);
			}else
			{
				solve_light(in,&(in->mylight),in->Psun,0.0);
			}

		}

		extracted_through_contacts=0.0;
	}else
	if (local_time<t1)
	{
		if (in->pulse_light==TRUE)
		{
			solve_light(in,&(in->mylight),in->pulse_sun,0.0);
		}else
		{
		V=in->pulse_Vss;
		}
	}else
	{
		if (in->pulse_light==TRUE)
		{
			solve_light(in,&(in->mylight),0.0,0.0);
		}else
		{
		V=V0;
		}
	}


	//solve_all(in);

	double i0=newton_celiv(in,V,TRUE);


	if (get_dump_status(dump_print_text)==TRUE)
	{
		printf("pulse time=%e step=%d %.1e ",in->time,step,in->last_error);
		char out[100];
		sprintf(out,"Vtot=%lf current = %e mA (%e A/m^2)\n",V,get_I(in)/1e-3,get_J(in));
		printdebug(out);
	}

	ittr++;

	plot_now(in,"plot");

	//inter_append(&pulseout,in->time,fabs(get_I(in)));

	inter_append(&out_j,in->time,i0);
	inter_append(&out_v,in->time,V);
	inter_append(&out_mue,in->time,get_avg_mue(in));
	inter_append(&out_muh,in->time,get_avg_muh(in));
	inter_append(&out_mue_c,in->time,in->mun[0]);
	inter_append(&out_muh_c,in->time,in->mup[0]);
	inter_append(&out_G,in->time,in->Gn[0]);
	inter_append(&lost_charge,in->time,extracted_through_contacts-fabs(get_extracted_n(in)+get_extracted_p(in))/2.0);

	device_timestep(in);
	step++;

	if (in->time>(((double)in->pulse_repeat)*t2)) break;
	if (local_time>t2) local_time=0.0;
	local_time+=in->dt;

	if (in->stop==TRUE)
	{
		break;

	}


}while(1);
char outpath[1000];
sprintf(outpath,"%s%s",in->outputpath,"pulse_j.dat");
inter_save(&out_j,outpath);

sprintf(outpath,"%s%s",in->outputpath,"pulse_j_norm.dat");
inter_sub(&out_j,out_j.data[0]);
inter_save(&out_j,outpath);


sprintf(outpath,"%s%s",in->outputpath,"pulse_v.dat");
inter_save(&out_v,outpath);

sprintf(outpath,"%s%s",in->outputpath,"pulse_mue.dat");
inter_save(&out_mue,outpath);

sprintf(outpath,"%s%s",in->outputpath,"pulse_muh.dat");
inter_save(&out_muh,outpath);

sprintf(outpath,"%s%s",in->outputpath,"pulse_mue_c.dat");
inter_save(&out_mue_c,outpath);

sprintf(outpath,"%s%s",in->outputpath,"pulse_muh_c.dat");
inter_save(&out_muh_c,outpath);

sprintf(outpath,"%s%s",in->outputpath,"pulse_G.dat");
inter_save(&out_G,outpath);

sprintf(outpath,"%s%s",in->outputpath,"pulse_lost_charge.dat");
inter_save(&lost_charge,outpath);

//inter_get(&out_j,in->pulse_time_j1)-inter_get(&out_j,in->pulse_time_j0)


in->go_time=FALSE;

inter_free(&out_G);
inter_free(&out_muh);
inter_free(&out_mue);
inter_free(&out_muh_c);
inter_free(&out_mue_c);
inter_free(&out_j);
inter_free(&out_v);
//</clean>
}
